//Copyright Christopher Marx & Tim Wiegers, Virtueller Campus Rheinland-Pfalz (VCRP), https://www.vcrp.de
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.



jQuery(document).ready(function ($) {

	"use strict";
	var Memory = {

		init: function (cards) {
			this.$game = $("#game-grid");
			this.$modal = $(".modal-grid");
			this.$overlay = $(".modal-overlay-grid");
			this.$restartButton = $("button#restart-grid");
			this.$more_infosButton = $("button.more-infos-grid");
			this.$moves = $(".num-moves");
			this.numMoves = 0;
			this.won = 0;
			this.moreInfos = 0;
			this.cardsArray = cards;
			this.shuffleCards(this.cardsArray);
			this.setup();
			this.binding();
			this.seconds = 0;
			this.minutes = 0;
			setTimeout(function () {
				Memory.countup();
			}, 1000);

		},

		shuffleCards: function (cardsArray) {
			this.$cards = $(this.shuffle(this.cardsArray));
		},

		setup: function () {
			if (this.moreInfos === 0) {
				this.html = this.buildHTML();

				if (document.getElementById('wrap-grid')) {
					document.getElementById('wrap-grid').style.height = "100vh";
				}

			} else {
				this.html = this.weitereInformationen();
				document.getElementById('game-grid').style.display = "block";
				document.getElementById('game-grid').style.padding = "25px";
				document.getElementById('wrap-grid').style.height = "auto";
				document.getElementById('weitere-infos-headline').style.display = "block";
				document.getElementById('anweisung').style.display = "none";
				document.getElementById('spielstand').style.display = "none";
			}

			this.$game.html(this.html);
			this.$memoryCards = $(".card");
			this.paused = false;
			this.guess = null;
		},

		binding: function () {
			this.$memoryCards.on("click", this.cardClicked);
			this.$restartButton.on("click", $.proxy(this.reset, this));
			this.$more_infosButton.on("click", $.proxy(this.more_infos, this));
		},

		cardClicked: function () {
			var _ = Memory;
			var $card = $(this);
			if (!_.paused && !$card.find(".inside").hasClass("matched") && !$card.find(".inside").hasClass("picked")) {
				$card.find(".inside").addClass("picked");
				if (!_.guess) {
					_.guess = $(this).attr("data-id");
				} else if (((_.guess - $(this).attr("data-id")) === 5) || (($(this).attr("data-id") - _.guess) === 5) && !$(this).hasClass("picked")) {
					$(".picked").addClass("matched");
					_.guess = null;
					Memory.gameCounterPlusOne();
				} else {
					_.guess = null;
					_.paused = true;
					Memory.gameCounterPlusOne();

				}
				if ($(".matched").length === $(".card").length) {

					_.win();

				}
			} else if (_.paused) {
				$(".picked").removeClass("picked");
				_.paused = false;
				$card.find(".inside").addClass("picked");
				if (!_.guess) {
					_.guess = $(this).attr("data-id");
				} else if (((_.guess - $(this).attr("data-id")) === 5) || (($(this).attr("data-id") - _.guess) === 5) && !$(this).hasClass("picked")) {
					$(".picked").addClass("matched");
					_.guess = null;
					Memory.gameCounterPlusOne();
				} else {
					_.guess = null;
					_.paused = true;
					Memory.gameCounterPlusOne();

				}
				if ($(".matched").length === $(".card").length) {

					_.win();

				}
			}
		},

		win: function () {
			if (this.seconds < 10) {
				this.seconds = '0' + this.seconds;
			}
			if (this.minutes < 10) {
				this.minutes = '0' + this.minutes;
			}
			this.numMoves = '<h4>Du hast in ' + this.numMoves + ' Zügen und in ' + this.minutes + ':' + this.seconds + ' Minuten gewonnen!</h4>';
			this.moveCounterUpdate = document.getElementById("winner-grid").innerHTML = this.numMoves;
			this.paused = true;
			this.won = 1;
			setTimeout(function () {
				Memory.showModal();
				Memory.$game.fadeOut();
			}, 3000);

		},

		showModal: function () {
			this.$overlay.show();
			this.$modal.fadeIn("slow");
		},

		hideModal: function () {
			this.$overlay.hide();
			this.$modal.hide();
		},

		reset: function () {
			window.location.reload(false);

		},

		more_infos: function () {

			this.hideModal();
			this.moreInfos = 1;
			this.setup();

		},

		gameCounterPlusOne: function () {

			this.numMoves = this.numMoves + 1;
			this.moveCounterUpdate = document.getElementById("num-moves").innerHTML = 'Züge: ' + this.numMoves;
		},

		countup: function () {

			if (document.getElementById("num-moves")) {

				if (this.seconds === 59) {
					this.seconds = -1;
					this.minutes = this.minutes + 1;
				}

				if (this.minutes === 59) {
					this.minutes = -1;
				}
				this.seconds++;
				if (this.won === 0) {
					setTimeout(function () {
						Memory.countup();
					}, 1000);

					if (this.seconds < 10) {
						this.moveCounterUpUpdate = document.getElementById("seconds").innerHTML = ':0' + this.seconds;
					} else {
						this.moveCounterUpUpdate = document.getElementById("seconds").innerHTML = ':' + this.seconds;
					}

					if (this.minutes < 10) {
						this.moveCounterUpUpdate = document.getElementById("minutes").innerHTML = 'Zeit: 0' + this.minutes;
					} else {
						this.moveCounterUpUpdate = document.getElementById("minutes").innerHTML = 'Zeit: ' + this.minutes;
					}

				}
			}
		},




		shuffle: function (array) {
			var counter = array.length,
				temp, index;

			while (counter > 0) {

				index = Math.floor(Math.random() * counter);

				counter--;

				temp = array[counter];
				array[counter] = array[index];
				array[index] = temp;
			}
			return array;
		},

		buildHTML: function () {
			var frag = '';
			this.$cards.each(function (k, v) {
				frag += '<div class="card" data-id="' + v.id + '"><div class="inside"><div class="front"><img src="https://www.oer-at-rlp.de/wp-content/plugins/vcrp-memoery/' + v.img + '" alt="' + v.name + '" /></div><div class="back"><img src="https://www.oer-at-rlp.de/wp-content/plugins/vcrp-memoery/js/memoery_tile_back.png" alt="OER Logo" /></div></div></div>';
			});

			return frag;
		},

		weitereInformationen: function () {
			var frag = '';

			frag = '<h4>Was sind Creative Commons Lizenzen?</h4><p>Noch vor wenigen Jahren mussten sich Privatpersonen nur wenige Sorgen darum machen, aus Versehen gegen das Urheberrecht zu verstoßen. Inzwischen hat sich das jedoch geändert und es stellt sich schwierig dar, durch den Tag zu kommen, ohne gegen das Urheberrecht zu verstoßen – schließlich werden alle im Internet angeschauten Daten erst einmal auf den eigenen Rechner „zwischenkopiert“, um dargestellt werden zu können. Aus diesem Grund hat die Creative Commons Corporation das Urheberrecht in kleine Bausteine verpackt, die untereinander kombiniert werden können. Die Creative Commons wird deshalb oft auch als Jedermann-Lizenz bezeichnet. Sie brauchen also keinen Abschluss in Jura, um mit Lizenzen zu arbeiten und nachts trotzdem noch ruhig schlafen zu können.</p><h4>CC-0 <img src="https://www.oer-at-rlp.de/wp-content/plugins/vcrp-memoery/js/memoery_cc0_icon_front.png" style="margin-left:15px; margin-top:-11px" width="80" alt="CC-0 Symbol" /></h4><p>Die Creative Commons Zero ist die Option, die wir im deutschen Urheberrecht am nächsten als Gemeinfreiheit, bzw. Public Domain beschreiben können, ohne dass der Ersteller erst 70 Jahre lang tot sein muss. In Deutschland ist es nicht möglich, sein Urheberrecht komplett abzutreten. Die drei Urheberpersönlichkeitsrechte sind nicht übertragbar. Es werden vom Urheber praktisch keine Einschränkungen erhoben. Sie dürfen Teilen, Bearbeiten, ALLES!</p><h4>CC-BY <img src="https://www.oer-at-rlp.de/wp-content/plugins/vcrp-memoery/js/memoery_ccby_icon_front.png" style="margin-left:15px; margin-top:-11px" width="80" alt="CC-0 Symbol" /></h4><p>Ein weiteres sehr offenes Creative Commons Modul ist CC BY – also das Modell mit Namensnennung der erstellenden Person. Sie dürfen das Werk teilen, bearbeiten, frei weiterlizenzieren, müssen jedoch den Namen des Urhebers nennen und auf eigene Weiterbearbeitungen hinweisen. </p><h4>CC-BY-SA <img src="https://www.oer-at-rlp.de/wp-content/plugins/vcrp-memoery/js/memoery_ccsa_icon_front.png" style="margin-left:15px; margin-top:-11px" width="80" alt="CC-0 Symbol" /></h4><p>Eine weitere recht offene Lizenz ist die CC-BY-SA-Lizenz. SA steht für „Share Alike“, was bedeutet, dass das Medium, selbst wenn es bearbeitet wurde, nur unter derselben Lizenz weitergegeben werden darf. Sie dürfen das Medium teilen, bearbeiten, müssen jedoch den Namen des Urhebers nennen und wie schon gesagt, das Medium unter gleicher Lizenz weitergeben. </p><h4>CC-BY-NC <img src="https://www.oer-at-rlp.de/wp-content/plugins/vcrp-memoery/js/memoery_ccnc_icon_front.png" style="margin-left:15px; margin-top:-11px" width="80" alt="CC-0 Symbol" /></h4><p>Die NC-Lizenz wird in OER-Kreisen schon recht kritisch gesehen – nicht jeder OER-Verfechter sieht Medien mit einer NC-Lizenz auch wirklich noch als offene Bildungsressource an. Die NC-Lizenz steht für Non-Commercial und untersagt daher die kommerzielle Nutzung des Mediums. Viele Personen wollen ihr Werk zwar gerne für den richtigen Zweck offen zur Verfügung stellen, wollen aber natürlich nicht, dass andere Personen mit den hart erarbeiteten Ressourcen Geld verdienen. Dabei wird allerdings oft vergessen, was diese Lizenz ausschließt. Wo fängt eine kommerzielle Nutzung an? Dürfen diese Materialien in einer staatlichen aber nicht in einer Privatschule verwendet werden? Dürfen Honorardozentinnen und Dozenten die Materialien in ihrer Lehre nicht einsetzen, weil die Ressource ihnen hilft, Geld zu verdienen? Insgesamt wird von vielen OER-Experten geraten, dass eine NC-Lizenz nur dann eingesetzt werden sollte, wenn Sie vorhaben, die Ausübung der Lizenzbedingungen auch wirklich durchzufechten. Ansonsten nutzen die Personen, die sich eh nicht um Lizenzverstöße kümmern, ihre Materialien ungeahndet und die Personen, die rechtstreu sind, werden benachteiligt.</p><h4>CC-BY-ND <img src="https://www.oer-at-rlp.de/wp-content/plugins/vcrp-memoery/js/memoery_ccnd_icon_front.png" style="margin-left:15px; margin-top:-11px" width="80" alt="CC-0 Symbol" /></h4><p>Die CC-BY-ND-Lizenz ist das geschlossenste aller Creative Commons Lizenzmodule. ND steht für No-Derivatives, d.h. es ist nicht erlaubt, Veränderungen am Content vorzunehmen. Das verstößt natürlich gegen die Prinzipien der Offenheit und ist somit ganz eindeutig keine OER mehr.</p><h4>Kombination von CC-Modulen</h4><p>Die Creative Commons ermöglicht es, einige Lizenzmodule zu kombinieren. Welche Module mit welchen kompatibel sind, können Sie in folgender Grafik erfahren:</p><img src="https://www.oer-at-rlp.de/wp-content/plugins/vcrp-memoery/js/CC_License_Compatibility_Chart.png" width="100%" alt="CC Lizenzkompatibilitäten" /><h4>Lizenzgenerator und Lizenzhinweisgenerator</h4><p>Es ist gar nicht so schwer, Creative Commons Lizenzen zu erstellen.Die Wahl einer passenden Lizenz wird durch den Lizenzgenerator auf der Creative Commons-Website (<a href="https://creativecommons.org/choose/?lang=de" target="_blank">https://creativecommons.org/choose/?lang=de</a>) vereinfacht. Das Schreiben von Hinweistexten zu Bildern aus der Wikipedia oder dem freien Medienarchiv Wikimedia Commons wird durch den Lizenzhinweisgenerator von Wikimedia Deutschland (<a href="https://lizenzhinweisgenerator.de/" target="_blank">https://lizenzhinweisgenerator.de/</a>) abgenommen.<br><button id="restart-grid" onclick="window.location.reload(false);">Nochmal spielen?</button>';

			return frag;
		}

	};

	var cards = [{
			name: "Gemeinfrei!",
			id: 1,
			img: "js/memoery_cc0_front.png"
		}, {
			name: "Namensnennung erforderlich",
			id: 2,
			img: "js/memoery_ccby_front.png"
		}, {
			name: "Nicht für kommerzielle Zwecke nutzbar",
			id: 3,
			img: "js/memoery_ccnc_front.png"
		}, {
			name: "Veränderungen nicht erlaubt",
			id: 4,
			img: "js/memoery_ccnd_front.png"
		}, {
			name: "Weitergabe nur unter Bedingungen der Ausgabelizenz",
			id: 5,
			img: "js/memoery_ccsa_front.png"
		}, {
			name: "CC-0",
			id: 6,
			img: "js/memoery_cc0_icon_front.png"
		},

		{
			name: "CC-BY",
			id: 7,
			img: "js/memoery_ccby_icon_front.png"
		},

		{
			name: "CC-NC",
			id: 8,
			img: "js/memoery_ccnc_icon_front.png"
		},

		{
			name: "CC-ND",
			id: 9,
			img: "js/memoery_ccnd_icon_front.png"
		},

		{
			name: "CC-SA",
			id: 10,
			img: "js/memoery_ccsa_icon_front.png"
		},
	];

	Memory.init(cards);


});
